@@ -1,7 +1,10 @@
-// Copyright 2007-2019 David Robillard <d@drobilla.net>
+// Copyright 2007-2025 David Robillard <d@drobilla.net>
 // SPDX-License-Identifier: ISC
 
 #include "lilv_internal.h"
+#include "log.h"
+#include "node_hash.h"
+#include "query.h"
 
 #ifdef LILV_DYN_MANIFEST
 #  include "dylib.h"
@@ -10,11 +13,11 @@
 
 #include <lilv/lilv.h>
 #include <lv2/core/lv2.h>
-#include <lv2/ui/ui.h>
 #include <serd/serd.h>
 #include <sord/sord.h>
 #include <zix/tree.h>
 
+#include <assert.h>
 #include <math.h>
 #include <stdarg.h>
 #include <stdbool.h>
@@ -23,9 +26,6 @@
 #include <stdlib.h>
 #include <string.h>
 
-#define NS_DOAP (const uint8_t*)"http://usefulinc.com/ns/doap#"
-#define NS_FOAF (const uint8_t*)"http://xmlns.com/foaf/0.1/"
-
 static void
 lilv_plugin_init(LilvPlugin* plugin, LilvNode* bundle_uri)
 {
@@ -40,10 +40,9 @@ lilv_plugin_init(LilvPlugin* plugin, LilvNode* bundle_uri)
   plugin->num_ports    = 0;
   plugin->loaded       = false;
   plugin->parse_errors = false;
-  plugin->replaced     = false;
 }
 
-/** Ownership of `uri` and `bundle` is taken */
+// Ownership of `uri` and `bundle` is taken
 LilvPlugin*
 lilv_plugin_new(LilvWorld* world, LilvNode* uri, LilvNode* bundle_uri)
 {
@@ -104,35 +103,13 @@ lilv_plugin_free(LilvPlugin* plugin)
   free(plugin);
 }
 
-static LilvNode*
-lilv_plugin_get_one(const LilvPlugin* plugin,
-                    const SordNode*   subject,
-                    const SordNode*   predicate)
+const SordNode*
+lilv_plugin_get_unique_internal(const LilvPlugin* const plugin,
+                                const SordNode* const   subject,
+                                const SordNode* const   predicate)
 {
-  /* TODO: This is slower than it could be in some cases, but it's simpler to
-     use the existing i18n code. */
-
-  SordIter* stream =
-    lilv_world_query_internal(plugin->world, subject, predicate, NULL);
-
-  LilvNodes* nodes =
-    lilv_nodes_from_stream_objects(plugin->world, stream, SORD_OBJECT);
-
-  if (nodes) {
-    LilvNode* value = lilv_node_duplicate(lilv_nodes_get_first(nodes));
-    lilv_nodes_free(nodes);
-    return value;
-  }
-
-  return NULL;
-}
-
-LilvNode*
-lilv_plugin_get_unique(const LilvPlugin* plugin,
-                       const SordNode*   subject,
-                       const SordNode*   predicate)
-{
-  LilvNode* ret = lilv_plugin_get_one(plugin, subject, predicate);
+  const SordNode* const ret =
+    lilv_world_get_unique(plugin->world, subject, predicate);
   if (!ret) {
     LILV_ERRORF("No value found for (%s %s ...) property\n",
                 sord_node_get_string(subject),
@@ -141,44 +118,49 @@ lilv_plugin_get_unique(const LilvPlugin* plugin,
   return ret;
 }
 
-static void
-lilv_plugin_load(LilvPlugin* plugin)
+LilvNode*
+lilv_plugin_get_unique(const LilvPlugin* plugin,
+                       const SordNode*   subject,
+                       const SordNode*   predicate)
 {
-  SordNode*       bundle_uri_node  = plugin->bundle_uri->node;
-  const SerdNode* bundle_uri_snode = sord_node_to_serd_node(bundle_uri_node);
+  return lilv_node_new_from_node(
+    plugin->world, lilv_plugin_get_unique_internal(plugin, subject, predicate));
+}
 
-  SerdEnv*    env = serd_env_new(bundle_uri_snode);
-  SerdReader* reader =
-    sord_new_reader(plugin->world->model, env, SERD_TURTLE, bundle_uri_node);
+static void
+load_prototypes(LilvPlugin* const plugin)
+{
+  NodeHash* const prots =
+    lilv_hash_from_matches(plugin->world->model,
+                           plugin->plugin_uri->node,
+                           plugin->world->uris.lv2_prototype,
+                           NULL,
+                           NULL);
+  if (!prots) {
+    return;
+  }
 
-  SordModel* prots = lilv_world_filter_model(plugin->world,
-                                             plugin->world->model,
-                                             plugin->plugin_uri->node,
-                                             plugin->world->uris.lv2_prototype,
-                                             NULL,
-                                             NULL);
-  SordModel* skel  = sord_new(plugin->world->world, SORD_SPO, false);
-  SordIter*  iter  = sord_begin(prots);
-  for (; !sord_iter_end(iter); sord_iter_next(iter)) {
-    const SordNode* t         = sord_iter_get_node(iter, SORD_OBJECT);
-    LilvNode*       prototype = lilv_node_new_from_node(plugin->world, t);
+  SordModel* skel = sord_new(plugin->world->world, SORD_SPO, false);
 
-    lilv_world_load_resource(plugin->world, prototype);
+  NODE_HASH_FOREACH (p, prots) {
+    const SordNode* prototype = lilv_node_hash_get(prots, p);
+
+    lilv_world_load_resource_internal(plugin->world, prototype);
 
     SordIter* statements =
-      sord_search(plugin->world->model, prototype->node, NULL, NULL, NULL);
+      sord_search(plugin->world->model, prototype, NULL, NULL, NULL);
     FOREACH_MATCH (statements) {
       SordQuad quad;
       sord_iter_get(statements, quad);
       quad[0] = plugin->plugin_uri->node;
       sord_add(skel, quad);
     }
-
     sord_iter_free(statements);
-    lilv_node_free(prototype);
   }
-  sord_iter_free(iter);
 
+  lilv_node_hash_free(prots, plugin->world->world);
+
+  SordIter* iter = NULL;
   for (iter = sord_begin(skel); !sord_iter_end(iter); sord_iter_next(iter)) {
     SordQuad quad;
     sord_iter_get(iter, quad);
@@ -186,7 +168,17 @@ lilv_plugin_load(LilvPlugin* plugin)
   }
   sord_iter_free(iter);
   sord_free(skel);
-  sord_free(prots);
+}
+
+static void
+lilv_plugin_load(LilvPlugin* plugin)
+{
+  load_prototypes(plugin);
+
+  SordNode*   bundle_node = plugin->bundle_uri->node;
+  SerdEnv*    env         = serd_env_new(sord_node_to_serd_node(bundle_node));
+  SerdReader* reader =
+    sord_new_reader(plugin->world->model, env, SERD_TURTLE, bundle_node);
 
   // Parse all the plugin's data files into RDF model
   SerdStatus st = SERD_SUCCESS;
@@ -194,7 +186,7 @@ lilv_plugin_load(LilvPlugin* plugin)
     const LilvNode* data_uri = lilv_nodes_get(plugin->data_uris, i);
 
     serd_env_set_base_uri(env, sord_node_to_serd_node(data_uri->node));
-    st = lilv_world_load_file(plugin->world, reader, data_uri);
+    st = lilv_world_load_file(plugin->world, reader, data_uri->node);
     if (st > SERD_FAILURE) {
       break;
     }
@@ -219,10 +211,11 @@ lilv_plugin_load(LilvPlugin* plugin)
       const SordNode* bundle = plugin->dynmanifest->bundle->node;
       serd_env_set_base_uri(env, sord_node_to_serd_node(bundle));
       FILE* fd = tmpfile();
+      assert(fd);
       get_data_func(plugin->dynmanifest->handle,
                     fd,
                     lilv_node_as_string(plugin->plugin_uri));
-      rewind(fd);
+      fseek(fd, 0, SEEK_SET);
       serd_reader_add_blank_prefix(reader,
                                    lilv_world_blank_node_prefix(plugin->world));
       serd_reader_read_file_handle(
@@ -260,42 +253,45 @@ lilv_plugin_load_ports_if_necessary(const LilvPlugin* const_plugin)
     plugin->ports    = (LilvPort**)malloc(sizeof(LilvPort*));
     plugin->ports[0] = NULL;
 
-    SordIter* ports = lilv_world_query_internal(plugin->world,
-                                                plugin->plugin_uri->node,
-                                                plugin->world->uris.lv2_port,
-                                                NULL);
+    SordIter* ports = sord_search(plugin->world->model,
+                                  plugin->plugin_uri->node,
+                                  plugin->world->uris.lv2_port,
+                                  NULL,
+                                  NULL);
 
     FOREACH_MATCH (ports) {
       const SordNode* port = sord_iter_get_node(ports, SORD_OBJECT);
 
-      LilvNode* index =
-        lilv_plugin_get_unique(plugin, port, plugin->world->uris.lv2_index);
+      const SordNode* symbol = lilv_plugin_get_unique_internal(
+        plugin, port, plugin->world->uris.lv2_symbol);
 
-      LilvNode* symbol =
-        lilv_plugin_get_unique(plugin, port, plugin->world->uris.lv2_symbol);
+      const char* const symbol_str =
+        symbol ? (const char*)sord_node_get_string(symbol) : "";
 
-      if (!lilv_node_is_string(symbol) ||
-          !is_symbol((const char*)sord_node_get_string(symbol->node))) {
+      if (!symbol || sord_node_get_type(symbol) != SORD_LITERAL ||
+          !is_symbol(symbol_str)) {
         LILV_ERRORF("Plugin <%s> port symbol `%s' is invalid\n",
                     lilv_node_as_uri(plugin->plugin_uri),
-                    lilv_node_as_string(symbol));
-        lilv_node_free(symbol);
-        lilv_node_free(index);
+                    symbol_str);
         lilv_plugin_free_ports(plugin);
         break;
       }
 
-      if (!lilv_node_is_int(index)) {
+      const SordNode* index = lilv_plugin_get_unique_internal(
+        plugin, port, plugin->world->uris.lv2_index);
+
+      if (!index || sord_node_get_type(index) != SORD_LITERAL ||
+          !sord_node_equals(sord_node_get_datatype(index),
+                            plugin->world->uris.xsd_integer)) {
         LILV_ERRORF("Plugin <%s> port index is not an integer\n",
                     lilv_node_as_uri(plugin->plugin_uri));
-        lilv_node_free(symbol);
-        lilv_node_free(index);
         lilv_plugin_free_ports(plugin);
         break;
       }
 
-      uint32_t  this_index = lilv_node_as_int(index);
-      LilvPort* this_port  = NULL;
+      const char* const index_str  = (const char*)sord_node_get_string(index);
+      const uint32_t    this_index = (uint32_t)strtol(index_str, NULL, 10);
+      LilvPort*         this_port  = NULL;
       if (plugin->num_ports > this_index) {
         this_port = plugin->ports[this_index];
       } else {
@@ -307,15 +303,14 @@ lilv_plugin_load_ports_if_necessary(const LilvPlugin* const_plugin)
         plugin->num_ports = this_index + 1;
       }
 
-      // Havn't seen this port yet, add it to array
+      // Haven't seen this port yet, add it to array
       if (!this_port) {
-        this_port = lilv_port_new(
-          plugin->world, port, this_index, lilv_node_as_string(symbol));
+        this_port = lilv_port_new(plugin->world, port, this_index, symbol_str);
         plugin->ports[this_index] = this_port;
       }
 
-      SordIter* types = lilv_world_query_internal(
-        plugin->world, port, plugin->world->uris.rdf_a, NULL);
+      SordIter* types = sord_search(
+        plugin->world->model, port, plugin->world->uris.rdf_type, NULL, NULL);
       FOREACH_MATCH (types) {
         const SordNode* type = sord_iter_get_node(types, SORD_OBJECT);
         if (sord_node_get_type(type) == SORD_URI) {
@@ -328,9 +323,6 @@ lilv_plugin_load_ports_if_necessary(const LilvPlugin* const_plugin)
         }
       }
       sord_iter_free(types);
-
-      lilv_node_free(symbol);
-      lilv_node_free(index);
     }
     sord_iter_free(ports);
 
@@ -351,6 +343,7 @@ lilv_plugin_load_ports_if_necessary(const LilvPlugin* const_plugin)
 void
 lilv_plugin_load_if_necessary(const LilvPlugin* plugin)
 {
+  assert(plugin);
   if (!plugin->loaded) {
     lilv_plugin_load((LilvPlugin*)plugin);
   }
@@ -374,10 +367,11 @@ lilv_plugin_get_library_uri(const LilvPlugin* plugin)
   lilv_plugin_load_if_necessary((LilvPlugin*)plugin);
   if (!plugin->binary_uri) {
     // <plugin> lv2:binary ?binary
-    SordIter* i = lilv_world_query_internal(plugin->world,
-                                            plugin->plugin_uri->node,
-                                            plugin->world->uris.lv2_binary,
-                                            NULL);
+    SordIter* i = sord_search(plugin->world->model,
+                              plugin->plugin_uri->node,
+                              plugin->world->uris.lv2_binary,
+                              NULL,
+                              NULL);
     FOREACH_MATCH (i) {
       const SordNode* binary_node = sord_iter_get_node(i, SORD_OBJECT);
       if (sord_node_get_type(binary_node) == SORD_URI) {
@@ -407,8 +401,11 @@ lilv_plugin_get_class(const LilvPlugin* plugin)
   lilv_plugin_load_if_necessary((LilvPlugin*)plugin);
   if (!plugin->plugin_class) {
     // <plugin> a ?class
-    SordIter* c = lilv_world_query_internal(
-      plugin->world, plugin->plugin_uri->node, plugin->world->uris.rdf_a, NULL);
+    SordIter* c = sord_search(plugin->world->model,
+                              plugin->plugin_uri->node,
+                              plugin->world->uris.rdf_type,
+                              NULL,
+                              NULL);
     FOREACH_MATCH (c) {
       const SordNode* class_node = sord_iter_get_node(c, SORD_OBJECT);
       if (sord_node_get_type(class_node) != SORD_URI) {
@@ -443,8 +440,8 @@ lilv_plugin_get_value_internal(const LilvPlugin* plugin,
                                const SordNode*   predicate)
 {
   lilv_plugin_load_if_necessary(plugin);
-  return lilv_world_find_nodes_internal(
-    plugin->world, plugin->plugin_uri->node, predicate, NULL);
+  return lilv_nodes_from_matches(
+    plugin->world, plugin->plugin_uri->node, predicate, NULL, NULL);
 }
 
 bool
@@ -626,26 +623,27 @@ bool
 lilv_plugin_has_latency(const LilvPlugin* plugin)
 {
   lilv_plugin_load_if_necessary(plugin);
-  SordIter* ports = lilv_world_query_internal(plugin->world,
-                                              plugin->plugin_uri->node,
-                                              plugin->world->uris.lv2_port,
-                                              NULL);
+  SordIter* ports = sord_search(plugin->world->model,
+                                plugin->plugin_uri->node,
+                                plugin->world->uris.lv2_port,
+                                NULL,
+                                NULL);
 
   bool ret = false;
   FOREACH_MATCH (ports) {
     const SordNode* port = sord_iter_get_node(ports, SORD_OBJECT);
 
-    SordIter* prop =
-      lilv_world_query_internal(plugin->world,
-                                port,
-                                plugin->world->uris.lv2_portProperty,
-                                plugin->world->uris.lv2_reportsLatency);
+    SordIter* prop = sord_search(plugin->world->model,
+                                 port,
+                                 plugin->world->uris.lv2_portProperty,
+                                 plugin->world->uris.lv2_reportsLatency,
+                                 NULL);
 
-    SordIter* des =
-      lilv_world_query_internal(plugin->world,
+    SordIter* des = sord_search(plugin->world->model,
                                 port,
                                 plugin->world->uris.lv2_designation,
-                                plugin->world->uris.lv2_latency);
+                                plugin->world->uris.lv2_latency,
+                                NULL);
 
     const bool latent = !sord_iter_end(prop) || !sord_iter_end(des);
     sord_iter_free(prop);
@@ -667,11 +665,11 @@ lilv_plugin_get_port_by_property(const LilvPlugin* plugin,
   lilv_plugin_load_ports_if_necessary(plugin);
   for (uint32_t i = 0; i < plugin->num_ports; ++i) {
     LilvPort* port = plugin->ports[i];
-    SordIter* iter =
-      lilv_world_query_internal(plugin->world,
-                                port->node->node,
-                                plugin->world->uris.lv2_portProperty,
-                                port_property);
+    SordIter* iter = sord_search(plugin->world->model,
+                                 port->node->node,
+                                 plugin->world->uris.lv2_portProperty,
+                                 port_property,
+                                 NULL);
 
     const bool found = !sord_iter_end(iter);
     sord_iter_free(iter);
@@ -693,8 +691,11 @@ lilv_plugin_get_port_by_designation(const LilvPlugin* plugin,
   lilv_plugin_load_ports_if_necessary(plugin);
   for (uint32_t i = 0; i < plugin->num_ports; ++i) {
     LilvPort* port = plugin->ports[i];
-    SordIter* iter = lilv_world_query_internal(
-      world, port->node->node, world->uris.lv2_designation, designation->node);
+    SordIter* iter = sord_search(world->model,
+                                 port->node->node,
+                                 world->uris.lv2_designation,
+                                 designation->node,
+                                 NULL);
 
     const bool found =
       !sord_iter_end(iter) &&
@@ -743,8 +744,11 @@ lilv_plugin_has_feature(const LilvPlugin* plugin, const LilvNode* feature)
                                   NULL};
 
   for (const SordNode** pred = predicates; *pred; ++pred) {
-    if (lilv_world_ask_internal(
-          plugin->world, plugin->plugin_uri->node, *pred, feature->node)) {
+    if (sord_ask(plugin->world->model,
+                 plugin->plugin_uri->node,
+                 *pred,
+                 feature->node,
+                 NULL)) {
       return true;
     }
   }
@@ -766,20 +770,22 @@ LilvNodes*
 lilv_plugin_get_optional_features(const LilvPlugin* plugin)
 {
   lilv_plugin_load_if_necessary(plugin);
-  return lilv_world_find_nodes_internal(plugin->world,
-                                        plugin->plugin_uri->node,
-                                        plugin->world->uris.lv2_optionalFeature,
-                                        NULL);
+  return lilv_nodes_from_matches(plugin->world,
+                                 plugin->plugin_uri->node,
+                                 plugin->world->uris.lv2_optionalFeature,
+                                 NULL,
+                                 NULL);
 }
 
 LilvNodes*
 lilv_plugin_get_required_features(const LilvPlugin* plugin)
 {
   lilv_plugin_load_if_necessary(plugin);
-  return lilv_world_find_nodes_internal(plugin->world,
-                                        plugin->plugin_uri->node,
-                                        plugin->world->uris.lv2_requiredFeature,
-                                        NULL);
+  return lilv_nodes_from_matches(plugin->world,
+                                 plugin->plugin_uri->node,
+                                 plugin->world->uris.lv2_requiredFeature,
+                                 NULL,
+                                 NULL);
 }
 
 bool
@@ -792,10 +798,11 @@ lilv_plugin_has_extension_data(const LilvPlugin* plugin, const LilvNode* uri)
   }
 
   lilv_plugin_load_if_necessary(plugin);
-  return lilv_world_ask_internal(plugin->world,
-                                 plugin->plugin_uri->node,
-                                 plugin->world->uris.lv2_extensionData,
-                                 uri->node);
+  return sord_ask(plugin->world->model,
+                  plugin->plugin_uri->node,
+                  plugin->world->uris.lv2_extensionData,
+                  uri->node,
+                  NULL);
 }
 
 LilvNodes*
@@ -835,13 +842,11 @@ lilv_plugin_get_project(const LilvPlugin* plugin)
 {
   lilv_plugin_load_if_necessary(plugin);
 
-  SordNode* lv2_project =
-    sord_new_uri(plugin->world->world, (const uint8_t*)LV2_CORE__project);
-
-  SordIter* projects = lilv_world_query_internal(
-    plugin->world, plugin->plugin_uri->node, lv2_project, NULL);
-
-  sord_node_free(plugin->world->world, lv2_project);
+  SordIter* projects = sord_search(plugin->world->model,
+                                   plugin->plugin_uri->node,
+                                   plugin->world->uris.lv2_project,
+                                   NULL,
+                                   NULL);
 
   if (sord_iter_end(projects)) {
     sord_iter_free(projects);
@@ -859,29 +864,28 @@ lilv_plugin_get_author(const LilvPlugin* plugin)
 {
   lilv_plugin_load_if_necessary(plugin);
 
-  SordNode* doap_maintainer =
-    sord_new_uri(plugin->world->world, NS_DOAP "maintainer");
+  const SordNode* doap_maintainer = plugin->world->uris.doap_maintainer;
 
-  SordIter* maintainers = lilv_world_query_internal(
-    plugin->world, plugin->plugin_uri->node, doap_maintainer, NULL);
+  SordIter* maintainers = sord_search(plugin->world->model,
+                                      plugin->plugin_uri->node,
+                                      doap_maintainer,
+                                      NULL,
+                                      NULL);
 
   if (sord_iter_end(maintainers)) {
     sord_iter_free(maintainers);
 
     LilvNode* project = lilv_plugin_get_project(plugin);
     if (!project) {
-      sord_node_free(plugin->world->world, doap_maintainer);
       return NULL;
     }
 
-    maintainers = lilv_world_query_internal(
-      plugin->world, project->node, doap_maintainer, NULL);
+    maintainers = sord_search(
+      plugin->world->model, project->node, doap_maintainer, NULL, NULL);
 
     lilv_node_free(project);
   }
 
-  sord_node_free(plugin->world->world, doap_maintainer);
-
   if (sord_iter_end(maintainers)) {
     sord_iter_free(maintainers);
     return NULL;
@@ -894,41 +898,38 @@ lilv_plugin_get_author(const LilvPlugin* plugin)
 }
 
 static LilvNode*
-lilv_plugin_get_author_property(const LilvPlugin* plugin, const uint8_t* uri)
+lilv_plugin_get_author_property(const LilvPlugin* plugin, const SordNode* pred)
 {
   const SordNode* author = lilv_plugin_get_author(plugin);
-  if (author) {
-    SordWorld* sworld = plugin->world->world;
-    SordNode*  pred   = sord_new_uri(sworld, uri);
-    LilvNode*  ret    = lilv_plugin_get_one(plugin, author, pred);
-    sord_node_free(sworld, pred);
-    return ret;
-  }
-  return NULL;
+
+  return author ? lilv_node_from_object(plugin->world, author, pred) : NULL;
 }
 
 LilvNode*
 lilv_plugin_get_author_name(const LilvPlugin* plugin)
 {
-  return lilv_plugin_get_author_property(plugin, NS_FOAF "name");
+  return lilv_plugin_get_author_property(plugin, plugin->world->uris.foaf_name);
 }
 
 LilvNode*
 lilv_plugin_get_author_email(const LilvPlugin* plugin)
 {
-  return lilv_plugin_get_author_property(plugin, NS_FOAF "mbox");
+  return lilv_plugin_get_author_property(plugin, plugin->world->uris.foaf_mbox);
 }
 
 LilvNode*
 lilv_plugin_get_author_homepage(const LilvPlugin* plugin)
 {
-  return lilv_plugin_get_author_property(plugin, NS_FOAF "homepage");
+  return lilv_plugin_get_author_property(plugin,
+                                         plugin->world->uris.foaf_homepage);
 }
 
 bool
 lilv_plugin_is_replaced(const LilvPlugin* plugin)
 {
-  return plugin->replaced;
+  return lilv_node_hash_find(plugin->world->replaced,
+                             lilv_plugin_get_uri(plugin)->node) !=
+         lilv_node_hash_end(plugin->world->replaced);
 }
 
 LilvUIs*
@@ -936,44 +937,38 @@ lilv_plugin_get_uis(const LilvPlugin* plugin)
 {
   lilv_plugin_load_if_necessary(plugin);
 
-  SordNode* ui_ui_node =
-    sord_new_uri(plugin->world->world, (const uint8_t*)LV2_UI__ui);
-  SordNode* ui_binary_node =
-    sord_new_uri(plugin->world->world, (const uint8_t*)LV2_UI__binary);
+  LilvUIs* result = lilv_uis_new();
 
-  LilvUIs*  result = lilv_uis_new();
-  SordIter* uis    = lilv_world_query_internal(
-    plugin->world, plugin->plugin_uri->node, ui_ui_node, NULL);
+  SordIter* uis = sord_search(plugin->world->model,
+                              plugin->plugin_uri->node,
+                              plugin->world->uris.ui_ui,
+                              NULL,
+                              NULL);
 
   FOREACH_MATCH (uis) {
     const SordNode* ui = sord_iter_get_node(uis, SORD_OBJECT);
 
-    LilvNode* type =
-      lilv_plugin_get_unique(plugin, ui, plugin->world->uris.rdf_a);
-    LilvNode* binary =
-      lilv_plugin_get_one(plugin, ui, plugin->world->uris.lv2_binary);
+    const SordNode* const type =
+      lilv_world_get_unique(plugin->world, ui, plugin->world->uris.rdf_type);
+    const SordNode* binary =
+      lilv_world_get_unique(plugin->world, ui, plugin->world->uris.lv2_binary);
     if (!binary) {
-      binary = lilv_plugin_get_unique(plugin, ui, ui_binary_node);
+      binary =
+        lilv_world_get_unique(plugin->world, ui, plugin->world->uris.ui_binary);
     }
 
-    if (sord_node_get_type(ui) != SORD_URI || !lilv_node_is_uri(type) ||
-        !lilv_node_is_uri(binary)) {
-      lilv_node_free(binary);
-      lilv_node_free(type);
+    if (sord_node_get_type(ui) != SORD_URI ||
+        sord_node_get_type(type) != SORD_URI ||
+        sord_node_get_type(binary) != SORD_URI) {
       LILV_ERRORF("Corrupt UI <%s>\n", sord_node_get_string(ui));
       continue;
     }
 
-    LilvUI* lilv_ui = lilv_ui_new(
-      plugin->world, lilv_node_new_from_node(plugin->world, ui), type, binary);
-
+    LilvUI* const lilv_ui = lilv_ui_new(plugin->world, ui, type, binary);
     zix_tree_insert((ZixTree*)result, lilv_ui, NULL);
   }
   sord_iter_free(uis);
 
-  sord_node_free(plugin->world->world, ui_binary_node);
-  sord_node_free(plugin->world->world, ui_ui_node);
-
   if (lilv_uis_size(result) > 0) {
     return result;
   }
@@ -987,25 +982,28 @@ lilv_plugin_get_related(const LilvPlugin* plugin, const LilvNode* type)
 {
   lilv_plugin_load_if_necessary(plugin);
 
-  LilvWorld* const world   = plugin->world;
-  LilvNodes* const related = lilv_world_find_nodes_internal(
-    world, NULL, world->uris.lv2_appliesTo, lilv_plugin_get_uri(plugin)->node);
+  LilvWorld* const world = plugin->world;
 
-  if (!type) {
-    return related;
+  SordIter* const i = sord_search(world->applications,
+                                  NULL,
+                                  world->uris.lv2_appliesTo,
+                                  lilv_plugin_get_uri(plugin)->node,
+                                  NULL);
+  if (sord_iter_end(i)) {
+    return NULL;
   }
 
   LilvNodes* matches = lilv_nodes_new();
-  LILV_FOREACH (nodes, i, related) {
-    const LilvNode* node = (LilvNode*)lilv_collection_get((ZixTree*)related, i);
-    if (lilv_world_ask_internal(
-          world, node->node, world->uris.rdf_a, type->node)) {
+  FOREACH_MATCH (i) {
+    const SordNode* node = sord_iter_get_node(i, SORD_SUBJECT);
+    if (!type ||
+        sord_ask(world->model, node, world->uris.rdf_type, type->node, NULL)) {
       zix_tree_insert(
-        (ZixTree*)matches, lilv_node_new_from_node(world, node->node), NULL);
+        (ZixTree*)matches, lilv_node_new_from_node(world, node), NULL);
     }
   }
 
-  lilv_nodes_free(related);
+  sord_iter_free(i);
   return matches;
 }
 
@@ -1062,14 +1060,14 @@ lilv_plugin_write_description(LilvWorld*        world,
 
   // Write plugin description
   SordIter* plug_iter =
-    lilv_world_query_internal(world, subject->node, NULL, NULL);
+    sord_search(world->model, subject->node, NULL, NULL, NULL);
   sord_write_iter(plug_iter, writer);
 
   // Write port descriptions
   for (uint32_t i = 0; i < num_ports; ++i) {
     const LilvPort* port = plugin->ports[i];
     SordIter*       port_iter =
-      lilv_world_query_internal(world, port->node->node, NULL, NULL);
+      sord_search(world->model, port->node->node, NULL, NULL, NULL);
     sord_write_iter(port_iter, writer);
   }
 
@@ -1107,7 +1105,7 @@ lilv_plugin_write_manifest_entry(LilvWorld*        world,
     0,
     NULL,
     sord_node_to_serd_node(subject->node),
-    sord_node_to_serd_node(plugin->world->uris.rdf_a),
+    sord_node_to_serd_node(plugin->world->uris.rdf_type),
     sord_node_to_serd_node(plugin->world->uris.lv2_Plugin),
     0,
     0);
