@@ -1,7 +1,14 @@
-// Copyright 2007-2022 David Robillard <d@drobilla.net>
+// Copyright 2007-2025 David Robillard <d@drobilla.net>
 // SPDX-License-Identifier: ISC
 
 #include "lilv_internal.h"
+#include "log.h"
+#include "node_hash.h"
+#include "node_skimmer.h"
+#include "query.h"
+#include "string_util.h"
+#include "sys_util.h"
+#include "type_skimmer.h"
 
 #include <lilv/lilv.h>
 #include <lv2/atom/atom.h>
@@ -368,7 +375,7 @@ absolute_path(LV2_State_Map_Path_Handle handle, const char* state_path)
   return path;
 }
 
-/** Return a new features array with built-in features added to `features`. */
+// Return a new features array with built-in features added to `features`
 static const LV2_Feature**
 add_features(const LV2_Feature* const* features,
              const LV2_Feature*        map,
@@ -409,7 +416,7 @@ add_features(const LV2_Feature* const* features,
   return ret;
 }
 
-/// Return a normal path for a directory with a trailing separator
+// Return a normal path for a directory with a trailing separator
 static char*
 normal_dir(const char* path)
 {
@@ -618,17 +625,23 @@ new_state_from_model(LilvWorld*      world,
   state->uri             = lilv_node_new_from_node(world, node);
 
   // Get the plugin URI this state applies to
-  SordIter* i = sord_search(model, node, world->uris.lv2_appliesTo, 0, 0);
-  if (i) {
+  SordIter* i = NULL;
+  if (sord_ask(model, node, world->uris.rdf_type, world->uris.lv2_Plugin, 0)) {
+    // Loading plugin description as state (default state)
+    state->plugin_uri = lilv_node_new_from_node(world, node);
+
+    // Load plugin data if necessary
+    const LilvPlugin* const plugin =
+      lilv_plugins_get_by_uri(world->plugins, state->plugin_uri);
+    if (plugin) {
+      lilv_plugin_load_if_necessary(plugin);
+    }
+  } else if ((i = sord_search(model, node, world->uris.lv2_appliesTo, 0, 0))) {
     const SordNode* object = sord_iter_get_node(i, SORD_OBJECT);
     const SordNode* graph  = sord_iter_get_node(i, SORD_GRAPH);
     state->plugin_uri      = lilv_node_new_from_node(world, object);
     set_state_dir_from_model(state, graph);
     sord_iter_free(i);
-  } else if (sord_ask(
-               model, node, world->uris.rdf_a, world->uris.lv2_Plugin, 0)) {
-    // Loading plugin description as state (default state)
-    state->plugin_uri = lilv_node_new_from_node(world, node);
   } else {
     LILV_ERRORF("State %s missing lv2:appliesTo property\n",
                 sord_node_get_string(node));
@@ -688,8 +701,8 @@ new_state_from_model(LilvWorld*      world,
   sord_iter_free(ports);
 
   // Get properties
-  SordNode* statep     = sord_new_uri(world->world, USTR(LV2_STATE__state));
-  SordNode* state_node = sord_get(model, node, statep, NULL, NULL);
+  SordNode* state_node =
+    sord_get(model, node, world->uris.state_state, NULL, NULL);
   if (state_node) {
     SordIter* props = sord_search(model, state_node, 0, 0, 0);
     FOREACH_MATCH (props) {
@@ -727,7 +740,6 @@ new_state_from_model(LilvWorld*      world,
     sord_iter_free(props);
   }
   sord_node_free(world->world, state_node);
-  sord_node_free(world->world, statep);
 
   serd_free((void*)chunk.buf);
   sratom_free(sratom);
@@ -768,38 +780,72 @@ lilv_state_new_from_file(LilvWorld*      world,
     return NULL;
   }
 
-  uint8_t* const abs_path = (uint8_t*)zix_canonical_path(NULL, path);
+  char* const abs_path = zix_canonical_path(NULL, path);
   if (!abs_path) {
     return NULL;
   }
 
-  SerdNode    node   = serd_node_new_file_uri(abs_path, NULL, NULL, true);
-  SerdEnv*    env    = serd_env_new(&node);
-  SordModel*  model  = sord_new(world->world, SORD_SPO, false);
-  SerdReader* reader = sord_new_reader(model, env, SERD_TURTLE, NULL);
+  const uint8_t* file_path     = (const uint8_t*)abs_path;
+  const char*    dir_path      = abs_path;
+  char*          manifest_path = NULL;
+  if (zix_file_type(abs_path) == ZIX_FILE_TYPE_DIRECTORY) {
+    manifest_path = zix_path_join(NULL, dir_path, "manifest.ttl");
+    file_path     = (uint8_t*)manifest_path;
+  } else {
+    dir_path = zix_path_parent_path(path).data;
+  }
 
-  serd_reader_read_file(reader, (const uint8_t*)node.buf);
+  SordModel* model    = sord_new(world->world, SORD_SPO, false);
+  SerdNode   file_uri = serd_node_new_file_uri(file_path, NULL, NULL, true);
 
-  const SordNode* subject_node =
-    (subject) ? subject->node
-              : sord_node_from_serd_node(world->world, env, &node, NULL, NULL);
+  NodeHash*          plugins = NULL;
+  NodeHash*          presets = NULL;
+  TypeSkimmer* const skimmer = type_skimmer_new(world->world,
+                                                &world->uris,
+                                                &file_uri,
+                                                model,
+                                                &plugins,
+                                                &presets,
+                                                NULL,
+                                                NULL,
+                                                NULL,
+                                                NULL);
 
-  char* const dirname = zix_string_view_copy(NULL, zix_path_parent_path(path));
-  char* const real_path = zix_canonical_path(NULL, dirname);
-  char* const dir_path  = zix_path_join(NULL, real_path, NULL);
+  serd_reader_read_file(skimmer->base.reader, (const uint8_t*)file_uri.buf);
+
+  const SordNode* subject_node = subject ? subject->node : NULL;
+  if (!subject_node) {
+    if (presets && lilv_node_hash_size(presets) == 1U) {
+      subject_node = lilv_node_hash_get(presets, lilv_node_hash_begin(presets));
+    } else if (plugins && lilv_node_hash_size(plugins) == 1U) {
+      subject_node = lilv_node_hash_get(plugins, lilv_node_hash_begin(plugins));
+    }
+  }
+
+  // Load any rdfs:seeAlso files
+  NodeHash* const files = lilv_hash_from_matches(
+    model, subject_node, world->uris.rdfs_seeAlso, NULL, NULL);
+  NODE_HASH_FOREACH (f, files) {
+    const SordNode* const link      = lilv_node_hash_get(files, f);
+    const char* const     link_uri  = (const char*)sord_node_get_string(link);
+    char* const           link_path = lilv_file_uri_parse(link_uri, NULL);
+
+    serd_env_set_base_uri(skimmer->base.env, sord_node_to_serd_node(link));
+    serd_reader_read_file(skimmer->base.reader, (const uint8_t*)link_path);
+    lilv_free(link_path);
+  }
+  lilv_node_hash_free(files, world->world);
 
   LilvState* const state =
     new_state_from_model(world, map, model, subject_node, dir_path);
 
-  zix_free(NULL, dir_path);
-  zix_free(NULL, real_path);
-  zix_free(NULL, dirname);
-
-  serd_node_free(&node);
+  type_skimmer_free(skimmer);
+  lilv_node_hash_free(presets, world->world);
+  lilv_node_hash_free(plugins, world->world);
+  serd_node_free(&file_uri);
+  zix_free(NULL, manifest_path);
   zix_free(NULL, abs_path);
-  serd_reader_free(reader);
   sord_free(model);
-  serd_env_free(env);
   return state;
 }
 
@@ -823,24 +869,29 @@ lilv_state_new_from_string(LilvWorld* world, LV2_URID_Map* map, const char* str)
     return NULL;
   }
 
-  SerdNode    base   = SERD_NODE_NULL;
-  SerdEnv*    env    = serd_env_new(&base);
-  SordModel*  model  = sord_new(world->world, SORD_SPO | SORD_OPS, false);
-  SerdReader* reader = sord_new_reader(model, env, SERD_TURTLE, NULL);
+  SordModel* model = sord_new(world->world, SORD_SPO, false);
 
-  set_prefixes(env);
-  serd_reader_read_string(reader, USTR(str));
+  NodeSkimmer* skimmer = node_skimmer_new(world->world,
+                                          &SERD_NODE_NULL,
+                                          model,
+                                          SORD_OBJECT,
+                                          world->uris.pset_Preset,
+                                          world->uris.rdf_type,
+                                          false);
 
-  SordNode* o = sord_new_uri(world->world, USTR(LV2_PRESETS__Preset));
-  SordNode* s = sord_get(model, NULL, world->uris.rdf_a, o, NULL);
+  set_prefixes(skimmer->base.env);
+  serd_reader_read_string(skimmer->base.reader, USTR(str));
 
-  LilvState* state = new_state_from_model(world, map, model, s, NULL);
+  NodeHash* const nodes = node_skimmer_free(skimmer);
+  LilvState*      state = NULL;
+  if (nodes && lilv_node_hash_size(nodes) == 1U) {
+    const SordNode* const node =
+      lilv_node_hash_get(nodes, lilv_node_hash_begin(nodes));
+    state = new_state_from_model(world, map, model, node, NULL);
+  }
 
-  sord_node_free(world->world, s);
-  sord_node_free(world->world, o);
-  serd_reader_free(reader);
+  lilv_node_hash_free(nodes, world->world);
   sord_free(model);
-  serd_env_free(env);
 
   return state;
 }
@@ -1367,12 +1418,11 @@ try_unlink(const char* state_dir, const char* path)
 }
 
 static char*
-get_canonical_path(const LilvNode* const node)
+get_canonical_path(const uint8_t* const manifest_uri)
 {
-  char* const path      = lilv_node_get_path(node, NULL);
-  char* const real_path = zix_canonical_path(NULL, path);
-
-  free(path);
+  char* const path      = (char*)serd_file_uri_parse(manifest_uri, NULL);
+  char* const real_path = path ? zix_canonical_path(NULL, path) : NULL;
+  serd_free(path);
   return real_path;
 }
 
@@ -1384,31 +1434,39 @@ lilv_state_delete(LilvWorld* world, const LilvState* state)
     return -1;
   }
 
-  LilvNode*  bundle        = lilv_new_file_uri(world, NULL, state->dir);
-  LilvNode*  manifest      = lilv_world_get_manifest_uri(world, bundle);
-  char*      manifest_path = get_canonical_path(manifest);
-  const bool has_manifest =
+  LilvNode*      bundle        = lilv_new_file_uri(world, NULL, state->dir);
+  uint8_t* const manifest_uri  = lilv_manifest_uri(bundle->node);
+  char* const    manifest_path = get_canonical_path(manifest_uri);
+  const bool     has_manifest =
     manifest_path && zix_file_type(manifest_path) == ZIX_FILE_TYPE_REGULAR;
 
-  SordModel* model = sord_new(world->world, SORD_SPO, false);
+  const SerdNode manifest_node =
+    serd_node_from_string(SERD_URI, (const uint8_t*)manifest_uri);
 
-  if (has_manifest) {
+  SordModel* model    = sord_new(world->world, SORD_SPO, false);
+  NodeHash*  see_also = NULL;
+
+  if (state->uri && has_manifest) {
     // Read manifest into temporary local model
-    SerdEnv*    env = serd_env_new(sord_node_to_serd_node(manifest->node));
-    SerdReader* ttl = sord_new_reader(model, env, SERD_TURTLE, NULL);
-    serd_reader_read_file(ttl, USTR(manifest_path));
-    serd_reader_free(ttl);
-    serd_env_free(env);
+    NodeSkimmer* const skimmer = node_skimmer_new(world->world,
+                                                  &manifest_node,
+                                                  model,
+                                                  SORD_SUBJECT,
+                                                  state->uri->node,
+                                                  world->uris.rdfs_seeAlso,
+                                                  true);
+
+    serd_reader_read_file(skimmer->base.reader, USTR(manifest_path));
+    see_also = node_skimmer_free(skimmer);
   }
 
   if (state->uri) {
-    const SordNode* file =
-      sord_get(model, state->uri->node, world->uris.rdfs_seeAlso, NULL, NULL);
-    if (file) {
-      // Remove state file
-      const uint8_t* uri       = sord_node_get_string(file);
-      char*          path      = (char*)serd_file_uri_parse(uri, NULL);
-      char*          real_path = zix_canonical_path(NULL, path);
+    // Remove state rdfs:seeAlso files
+    NODE_HASH_FOREACH (s, see_also) {
+      const SordNode* node      = lilv_node_hash_get(see_also, s);
+      const uint8_t*  uri       = sord_node_get_string(node);
+      char* const     path      = (char*)serd_file_uri_parse(uri, NULL);
+      char* const     real_path = zix_canonical_path(NULL, path);
       if (real_path) {
         try_unlink(state->dir, real_path);
       }
@@ -1418,10 +1476,11 @@ lilv_state_delete(LilvWorld* world, const LilvState* state)
 
     // Remove any existing manifest entries for this state
     const char* state_uri_str = lilv_node_as_string(state->uri);
-    remove_manifest_entry(world->world, model, state_uri_str);
     remove_manifest_entry(world->world, world->model, state_uri_str);
   }
 
+  lilv_node_hash_free(see_also, world->world);
+
   // Drop bundle from model
   lilv_world_unload_bundle(world, bundle);
 
@@ -1458,17 +1517,15 @@ lilv_state_delete(LilvWorld* world, const LilvState* state)
     }
   } else {
     // Still something in the manifest, update and reload bundle
-    const SerdNode* manifest_node = sord_node_to_serd_node(manifest->node);
-    SerdEnv*        env           = serd_env_new(manifest_node);
-
-    write_manifest(world, env, model, manifest_node);
+    SerdEnv* env = serd_env_new(&manifest_node);
+    write_manifest(world, env, model, &manifest_node);
     lilv_world_load_bundle(world, bundle);
     serd_env_free(env);
   }
 
   sord_free(model);
   zix_free(NULL, manifest_path);
-  lilv_node_free(manifest);
+  zix_free(NULL, manifest_uri);
   lilv_node_free(bundle);
 
   return 0;
