@@ -758,14 +758,14 @@ lilv_world_compare_versions(LilvWorld* const      world,
                sord_node_get_string(old_bundle));
   } else if (cmp < 0) {
     LILV_WARNF("Ignoring version %d.%d of <%s> from <%s>\n",
-               old_version.minor,
-               old_version.micro,
-               sord_node_get_string(resource),
-               sord_node_get_string(old_bundle));
-    LILV_NOTEF("Newer version %d.%d loaded from <%s>\n",
                new_version.minor,
                new_version.micro,
+               sord_node_get_string(resource),
                sord_node_get_string(new_bundle));
+    LILV_NOTEF("Newer version %d.%d loaded from <%s>\n",
+               old_version.minor,
+               old_version.micro,
+               sord_node_get_string(old_bundle));
   }
   return cmp;
 }
@@ -843,6 +843,7 @@ lilv_world_load_bundle(LilvWorld* world, const LilvNode* bundle_uri)
           type_skimmer_free(skimmer);
           lilv_node_hash_free(specs, world->world);
           lilv_node_hash_free(plugins, world->world);
+          return;
         }
       }
     }
@@ -916,14 +917,35 @@ lilv_world_unload_bundle(LilvWorld* world, const LilvNode* bundle_uri)
     return 0;
   }
 
-  // Remove loaded files entries (the actual file data is dropped below)
+  // Collect loaded files from this bundle (the actual data is dropped below)
+  NodeHash* unload_files = lilv_node_hash_new(NULL);
   NODE_HASH_FOREACH (i, world->loaded_files) {
     const SordNode* const file = lilv_node_hash_get(world->loaded_files, i);
-    if (file && sord_node_equals(file, bundle_uri->node)) {
-      lilv_node_hash_remove(world->loaded_files, world->world, file);
+
+    size_t               file_len = 0U;
+    const uint8_t* const file_str =
+      sord_node_get_string_counted(file, &file_len);
+
+    size_t               bundle_len = 0U;
+    const uint8_t* const bundle_str =
+      sord_node_get_string_counted(bundle_uri->node, &bundle_len);
+
+    if (file && (sord_node_equals(file, bundle_uri->node) ||
+                 (file_len > bundle_len && !strncmp((const char*)file_str,
+                                                    (const char*)bundle_str,
+                                                    bundle_len)))) {
+      lilv_node_hash_insert(unload_files, sord_node_copy(file));
     }
   }
 
+  // Remove files from world records so they'll be read again if loaded
+  NODE_HASH_FOREACH (i, unload_files) {
+    const SordNode* const file = lilv_node_hash_get(unload_files, i);
+    lilv_node_hash_remove(world->loaded_files, world->world, file);
+  }
+
+  lilv_node_hash_free(unload_files, world->world);
+
   /* Remove any plugins in the bundle from the plugin list.  Since the
      application may still have a pointer to the LilvPlugin, it can not be
      destroyed here.  Instead, we move it to the zombie plugin list, so it
@@ -950,10 +972,13 @@ lilv_world_unload_bundle(LilvWorld* world, const LilvNode* bundle_uri)
 static void
 load_dir_entry(const char* dir, const char* name, void* data)
 {
-  LilvWorld* const world = (LilvWorld*)data;
-  char* const      path  = zix_path_join(NULL, dir, name);
-  if (zix_file_type(path) != ZIX_FILE_TYPE_DIRECTORY) {
-    LILV_WARNF("Skipping non-directory `%s' within path entry\n", path);
+  LilvWorld* const  world = (LilvWorld*)data;
+  char* const       path  = zix_path_join(NULL, dir, name);
+  const ZixFileType type  = zix_file_type(path);
+  if (type != ZIX_FILE_TYPE_DIRECTORY) {
+    if (type != ZIX_FILE_TYPE_NONE) {
+      LILV_WARNF("Skipping non-directory `%s' within path entry\n", path);
+    }
     free(path);
     return;
   }
@@ -975,9 +1000,10 @@ lilv_world_load_directory(LilvWorld* world, const char* dir_path)
 {
   char* const path = zix_expand_environment_strings(NULL, dir_path);
   if (path) {
-    if (zix_file_type(path) == ZIX_FILE_TYPE_DIRECTORY) {
+    const ZixFileType type = zix_file_type(path);
+    if (type == ZIX_FILE_TYPE_DIRECTORY) {
       zix_dir_for_each(path, world, load_dir_entry);
-    } else {
+    } else if (type != ZIX_FILE_TYPE_NONE) {
       LILV_WARNF("Skipping non-directory `%s' in path\n", path);
     }
     free(path);
