@@ -1,7 +1,7 @@
 // Copyright 2017-2019 David Robillard <d@drobilla.net>
 // SPDX-License-Identifier: ISC
 
-#include "lilv/lilvmm.hpp"
+#include <lilv/lilvmm.hpp>
 
 int
 main()
