@@ -1,20 +1,13 @@
 // Copyright 2011-2019 David Robillard <d@drobilla.net>
 // SPDX-License-Identifier: ISC
 
-#include "lv2/core/lv2.h"
+#include <lv2/core/lv2.h>
 
 #include <stdint.h>
 #include <stdlib.h>
 
 #define PLUGIN_URI "http://example.org/failed-instantiation"
 
-enum { TEST_INPUT = 0, TEST_OUTPUT = 1 };
-
-typedef struct {
-  float* input;
-  float* output;
-} Test;
-
 static void
 cleanup(LV2_Handle instance)
 {
