@@ -1,7 +1,7 @@
 // Copyright 2011-2018 David Robillard <d@drobilla.net>
 // SPDX-License-Identifier: ISC
 
-#include "lv2/core/lv2.h"
+#include <lv2/core/lv2.h>
 
 LV2_SYMBOL_EXPORT
 extern const char* msg;
