@@ -60,6 +60,7 @@ static TestContext*
 test_context_new(void)
 {
   TestContext* ctx = (TestContext*)calloc(1, sizeof(TestContext));
+  assert(ctx);
 
   lilv_test_uri_map_init(&ctx->uri_map);
 
@@ -305,6 +306,33 @@ test_instance_state(void)
   test_context_free(ctx);
 }
 
+static void
+test_default_state(void)
+{
+  TestContext* const      ctx        = test_context_new();
+  const LilvPlugin* const plugin     = load_test_plugin(ctx);
+  const LilvNode* const   plugin_uri = lilv_plugin_get_uri(plugin);
+
+  LilvState* const state =
+    lilv_state_new_from_world(ctx->env->world, &ctx->map, plugin_uri);
+
+  assert(lilv_state_get_num_properties(state) == 0);
+  assert(lilv_node_equals(lilv_state_get_uri(state), plugin_uri));
+
+  char* const bundle_path = lilv_file_uri_parse(
+    lilv_node_as_string(lilv_plugin_get_bundle_uri(plugin)), NULL);
+
+  LilvState* const bundle_state =
+    lilv_state_new_from_file(ctx->env->world, &ctx->map, NULL, bundle_path);
+  assert(bundle_state);
+  assert(lilv_state_equals(bundle_state, state));
+
+  lilv_state_free(bundle_state);
+  lilv_free(bundle_path);
+  lilv_state_free(state);
+  test_context_free(ctx);
+}
+
 static void
 test_equal(void)
 {
@@ -504,6 +532,7 @@ test_string_round_trip(void)
     lilv_state_new_from_string(ctx->env->world, &ctx->map, string);
 
   // Ensure they are equal
+  assert(restored);
   assert(lilv_state_equals(initial_state, restored));
 
   // Check that the restored state refers to the correct plugin
@@ -840,10 +869,8 @@ test_files_round_trip(void)
                           "state.ttl"));
 
   // Load updated state bundle and check that it differs from the others
-  char* const state_2_path = zix_path_join(NULL, bundle_2_path, "state.ttl");
-
   LilvState* state_2_loaded =
-    lilv_state_new_from_file(ctx->env->world, &ctx->map, NULL, state_2_path);
+    lilv_state_new_from_file(ctx->env->world, &ctx->map, NULL, bundle_2_path);
 
   assert(state_2_loaded);
   assert(!lilv_state_equals(state_1_1_loaded, state_2_loaded));
@@ -858,7 +885,6 @@ test_files_round_trip(void)
   cleanup_test_directories(dirs);
 
   lilv_state_free(state_2_loaded);
-  free(state_2_path);
   lilv_state_free(state_2);
   free(bundle_2_path);
   lilv_state_free(state_1_2_loaded);
@@ -926,6 +952,7 @@ test_world_round_trip(void)
   // Ensure the state loaded from the world matches
   LilvState* const restored =
     lilv_state_new_from_world(world, &ctx->map, state_node);
+  assert(restored);
   assert(lilv_state_equals(start_state, restored));
 
   // Unload state from world
@@ -1106,6 +1133,7 @@ int
 main(void)
 {
   test_instance_state();
+  test_default_state();
   test_equal();
   test_changed_plugin_data();
   test_changed_metadata();
